/******************************************************************************
 * HI 5323 Image Processing
 *
 * complex.h
 ******************************************************************************/

#ifndef __COMPLEX_H
#define __COMPLEX_H
 
#include <math.h>

/**
 * Complex
 */
class Complex
{
protected:

    float r, i;

public:

    /**
     * Constructor
     */
    Complex(float real=0.0, float imag=0.0) : r(real), i(imag)
    {
    };

    float real() const
    {
        return r;
    };
    float imag() const
    {
        return i;
    };

    /**
     * Magnitude
     */
    float abs() const
    {
        return sqrt(r * r + i * i);
    };
    /**
     * Magnitude squared
     */
    float norm() const
    {
        return r * r + i * i;
    };
    /**
     * Phase
     */
    float arg() const
    {
        return atan2(i , r);
    };

    /**
     * Complex conjugate
     */
    Complex conj() const
    {
        return Complex(r, -i);
    };
    Complex& conjEqu()
    {
        i = -i;
        return *this;
    };

    /**
     * Unary operators
     */
    Complex operator+() const
    {
        return *this;
    };
    Complex operator-() const
    {
        return Complex(-r, -i);
    };

    /**
     * Addition operators
     */
    Complex operator+(float a) const
    {
        return Complex(r + a, i);
    };
    Complex& operator+=(float a)
    {
        r += a;
        return *this;
    };

    Complex operator+(Complex z) const
    {
        return Complex(r + z.r, i + z.i);
    };
    Complex &operator+=(Complex z)
    {
        r += z.r; i+= z.i;
        return *this;
    };

    /**
     * Subtraction operators
     */
    Complex operator-(float a) const
    {
        return Complex(r - a, i);
    };
    Complex& operator-=(float a)
    {
        r -= a;
        return *this;
    };

    Complex operator-(Complex z) const
    {
        return Complex(r - z.r, i - z.i);
    };
    Complex& operator-=(Complex z)
    {
        r -= z.r;
        i-= z.i;
        return *this;
    };

    /**
     * Multiplication operators
     */
    Complex operator*(float a) const
    {
        return Complex(r * a, i * a);
    };
    Complex& operator*=(float a)
    {
        r *= a;
        i *= a;
        return *this;
    };
    Complex operator* (Complex z) const
    {
        return Complex(r * z.r - i * z.i, r * z.i + i * z.r);
    };
    Complex& operator*=(Complex z)
    {
        float tmp = r * z.r - i * z.i;
        i = r * z.i + i * z.r;
        r = tmp;
        return *this;
    };

    /**
     * Division operators
     */
    Complex operator/(float a) const
    {
        return Complex(r / a, i / a);
    };
    Complex& operator/=(float a)
    {
        r /= a;
        i /= a;
        return *this;
    };
                                   
    /**
     * Comparison operators
     */
    bool operator==(float a) const
    {
        return r == a && i == 0.0;
    };
    bool operator==(Complex z) const
    {
        return r == z.r && i == z.i;
    };
    bool operator!=(float a) const
    {
        return r != a || i != 0.0;
    };
    bool operator!=(Complex z) const
    {
        return r == z.r || i == z.i;
    };
};


inline Complex euler(float ang)
{
    return Complex(cos(ang), sin(ang));
};

inline Complex polar(float rad, float ang)
{
    return Complex(rad * cos(ang), rad * sin(ang));
};

inline Complex operator+(float a, const Complex &z)
{
    return Complex(a + z.real(), z.imag());
};

inline Complex operator-(float a, const Complex &z)
{
    return Complex(a - z.real(), -z.imag());
};

inline Complex operator*(float a, const Complex &z)
{
    return Complex(a * z.real(), a * z.imag());
};

inline bool operator==(float a, const Complex &z)
{
    return a == z.real() && z.imag() == 0.0;
};

inline bool operator!=(float a, const Complex &z)
{
    return a != z.real() || z.imag() == 0.0;
};

#endif
